@@ -2,7 +2,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = '0.12.7'
+VERSION = '0.12.9.1'
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = 'local'
@@ -21,6 +21,7 @@ DEFAULT_PORT_WEB_UI = 8080
 # host name for localhost
 LOCALHOST = 'localhost'
 LOCALHOST_IP = '127.0.0.1'
+LOCALHOST_HOSTNAME = 'localhost.localstack.cloud'
 
 # version of the Maven dependency with Java utility code
 LOCALSTACK_MAVEN_VERSION = '0.2.5'
@@ -137,3 +138,7 @@ INTERNAL_AWS_SECRET_ACCESS_KEY = '__internal_call__'
 
 # list of official docker images
 OFFICIAL_IMAGES = ['localstack/localstack', 'localstack/localstack-light', 'localstack/localstack-full']
+
+# s3 virtual host name
+S3_VIRTUAL_HOSTNAME = 's3.%s' % LOCALHOST_HOSTNAME
+S3_STATIC_WEBSITE_HOSTNAME = 's3-website.%s' % LOCALHOST_HOSTNAME
