@@ -21,7 +21,7 @@ from localstack.constants import TEST_AWS_ACCOUNT_ID
 from localstack.utils.aws import aws_stack, aws_responses
 from localstack.utils.common import (
     to_str, to_bytes, load_file, save_file, TMP_FILES, ensure_readable, short_uid, long_uid, json_safe,
-    mkdir, unzip, is_zip_file, run, first_char_to_lower, run_for_max_seconds,
+    mkdir, unzip, is_zip_file, run, run_safe, first_char_to_lower, run_for_max_seconds, parse_request_data,
     timestamp_millis, now_utc, safe_requests, FuncThread, isoformat_milliseconds, synchronized)
 from localstack.services.awslambda import lambda_executors
 from localstack.services.generic_proxy import RegionBackend
@@ -290,30 +290,36 @@ def process_apigateway_invocation(func_arn, path, payload, stage, api_id, header
                                   stage_variables={}, request_context={}, event_context={}):
     try:
         resource_path = resource_path or path
+        event = construct_invocation_event(method, resource_path, headers, payload, query_string_params)
         path_params = dict(path_params)
         fix_proxy_path_params(path_params)
-        event = {
-            'path': path,
-            'headers': dict(headers),
-            'multiValueHeaders': multi_value_dict_for_list(headers),
-            'pathParameters': path_params,
-            'body': payload,
-            'isBase64Encoded': False,
-            'resource': resource_path,
-            'httpMethod': method,
-            'queryStringParameters': query_string_params,
-            'multiValueQueryStringParameters': multi_value_dict_for_list(query_string_params),
-            'requestContext': request_context,
-            'stageVariables': stage_variables,
-        }
+        event['pathParameters'] = path_params
+        event['resource'] = resource_path
+        event['requestContext'] = request_context
+        event['stageVariables'] = stage_variables
         LOG.debug('Running Lambda function %s from API Gateway invocation: %s %s' % (func_arn, method or 'GET', path))
         asynchronous = not config.SYNCHRONOUS_API_GATEWAY_EVENTS
-        inv_result = run_lambda(event=event, context=event_context, func_arn=func_arn, asynchronous=asynchronous)
+        inv_result = run_lambda(func_arn=func_arn, event=event, context=event_context, asynchronous=asynchronous)
         return inv_result.result
     except Exception as e:
         LOG.warning('Unable to run Lambda function on API Gateway message: %s %s' % (e, traceback.format_exc()))
 
 
+def construct_invocation_event(method, path, headers, data, query_string_params={}):
+    query_string_params = query_string_params or parse_request_data(method, path, '')
+    event = {
+        'path': path,
+        'headers': dict(headers),
+        'multiValueHeaders': multi_value_dict_for_list(headers),
+        'body': data,
+        'isBase64Encoded': False,
+        'httpMethod': method,
+        'queryStringParameters': query_string_params,
+        'multiValueQueryStringParameters': multi_value_dict_for_list(query_string_params)
+    }
+    return event
+
+
 def process_sns_notification(func_arn, topic_arn, subscription_arn, message, message_id,
         message_attributes, unsubscribe_url, subject='',):
     event = {
@@ -338,7 +344,7 @@ def process_sns_notification(func_arn, topic_arn, subscription_arn, message, mes
             }
         }]
     }
-    inv_result = run_lambda(event=event, context={}, func_arn=func_arn, asynchronous=not config.SYNCHRONOUS_SNS_EVENTS)
+    inv_result = run_lambda(func_arn=func_arn, event=event, context={}, asynchronous=not config.SYNCHRONOUS_SNS_EVENTS)
     return inv_result.result
 
 
@@ -370,7 +376,7 @@ def process_kinesis_records(records, stream_name):
                         for rec in chunk
                     ]
                 }
-                run_lambda(event=event, context={}, func_arn=arn, asynchronous=not config.SYNCHRONOUS_KINESIS_EVENTS)
+                run_lambda(func_arn=arn, event=event, context={}, asynchronous=not config.SYNCHRONOUS_KINESIS_EVENTS)
     except Exception as e:
         LOG.warning('Unable to run Lambda function on Kinesis records: %s %s' % (e, traceback.format_exc()))
 
@@ -411,7 +417,7 @@ def start_lambda_sqs_listener():
         event = {'Records': records}
 
         # TODO implement retries, based on "RedrivePolicy.maxReceiveCount" in the queue settings
-        run_lambda(event=event, context={}, func_arn=lambda_arn, asynchronous=True, callback=delete_messages)
+        run_lambda(func_arn=lambda_arn, event=event, context={}, asynchronous=True, callback=delete_messages)
 
     def listener_loop(*args):
         while True:
@@ -548,7 +554,8 @@ def do_update_alias(arn, alias, version, description=None):
     return new_alias
 
 
-def run_lambda(event, context, func_arn, version=None, suppress_output=False, asynchronous=False, callback=None):
+def run_lambda(func_arn, event, context={}, version=None,
+        suppress_output=False, asynchronous=False, callback=None):
     region_name = func_arn.split(':')[3]
     region = LambdaRegion.get(region_name)
     if suppress_output:
@@ -883,7 +890,8 @@ def format_func_details(func_details, version=None, always_add_version=False):
         'RevisionId': func_version.get('RevisionId'),
         'State': 'Active',
         'LastUpdateStatus': 'Successful',
-        'PackageType': func_details.package_type
+        'PackageType': func_details.package_type,
+        'ImageConfig': func_details.image_config
     }
     if func_details.dead_letter_config:
         result['DeadLetterConfig'] = func_details.dead_letter_config
@@ -912,8 +920,9 @@ def forward_to_external_url(func_details, event, context, invocation_type):
     data = json.dumps(event) if isinstance(event, dict) else str(event)
     LOG.debug('Forwarding Lambda invocation to LAMBDA_FORWARD_URL: %s' % config.LAMBDA_FORWARD_URL)
     result = safe_requests.post(url, data, headers=headers)
-    LOG.debug('Received result from external Lambda endpoint (status %s): %s' % (
-        result.status_code, result.content))
+    content = run_safe(lambda: to_str(result.content)) or result.content
+    LOG.debug('Received result from external Lambda endpoint (status %s): %s' % (result.status_code, content))
+    result = aws_responses.requests_to_flask_response(result)
     result = lambda_executors.InvocationResult(result)
     return result
 
@@ -1034,7 +1043,8 @@ def create_function():
         func_details.memory_size = data.get('MemorySize')
         func_details.code_signing_config_arn = data.get('CodeSigningConfigArn')
         func_details.code = data['Code']
-        func_details.package_type = 'Zip'
+        func_details.package_type = data.get('PackageType') or 'Zip'
+        func_details.image_config = data.get('ImageConfig', {})
         func_details.set_dead_letter_config(data)
         result = set_function_code(func_details.code, lambda_name)
         if isinstance(result, Response):
@@ -1445,10 +1455,10 @@ def invoke_function(function):
 
     if invocation_type == 'RequestResponse':
         context = {'client_context': request.headers.get('X-Amz-Client-Context')}
-        result = run_lambda(asynchronous=False, func_arn=arn, event=data, context=context, version=qualifier)
+        result = run_lambda(func_arn=arn, event=data, context=context, asynchronous=False, version=qualifier)
         return _create_response(result)
     elif invocation_type == 'Event':
-        run_lambda(asynchronous=True, func_arn=arn, event=data, context={}, version=qualifier)
+        run_lambda(func_arn=arn, event=data, context={}, asynchronous=True, version=qualifier)
         return _create_response('', status_code=202)
     elif invocation_type == 'DryRun':
         # Assume the dry run always passes.
