@@ -29,6 +29,7 @@ from localstack.services.generic_proxy import GenericProxyHandler, ProxyListener
 from localstack.services.cloudformation import cloudformation_api
 from localstack.services.dynamodbstreams import dynamodbstreams_api
 from localstack.utils.analytics.profiler import log_duration
+from localstack.utils.cli import print_version
 
 # flag to indicate whether signal handlers have been set up already
 SIGNAL_HANDLERS_SETUP = False
@@ -234,13 +235,13 @@ def register_signal_handlers():
     SIGNAL_HANDLERS_SETUP = True
 
 
-def do_run(cmd, asynchronous, print_output=None, env_vars={}):
+def do_run(cmd, asynchronous, print_output=None, env_vars={}, auto_restart=False):
     sys.stdout.flush()
     if asynchronous:
         if config.DEBUG and print_output is None:
             print_output = True
         outfile = subprocess.PIPE if print_output else None
-        t = ShellCommandThread(cmd, outfile=outfile, env_vars=env_vars)
+        t = ShellCommandThread(cmd, outfile=outfile, env_vars=env_vars, auto_restart=auto_restart)
         t.start()
         TMP_THREADS.append(t)
         return t
@@ -359,6 +360,8 @@ def start_infra(asynchronous=False, apis=None):
             print('!WARNING! - Looks like you have configured $LAMBDA_REMOTE_DOCKER=0 - '
                   "please make sure to configure $HOST_TMP_FOLDER to point to your host's $TMPDIR")
 
+        print_version(is_in_docker)
+
         # apply patches
         patch_urllib3_connection_pool(maxsize=128)
         patch_instance_tracker_meta()
@@ -435,7 +438,10 @@ def do_start_infra(asynchronous, apis, is_in_docker):
         # ensure that all infra components are up and running
         check_infra(apis=apis)
         # restore persisted data
+        record_service_health('features:persistence', 'initializing' if config.DATA_DIR else 'disabled')
         persistence.restore_persisted_data(apis=apis)
+        if config.DATA_DIR:
+            record_service_health('features:persistence', 'initialized')
         return thread
 
     prepare_environment()
