@@ -7,11 +7,12 @@ from moto.s3.exceptions import S3ClientError
 from moto.s3bucket_path import utils as s3bucket_path_utils
 from localstack import config
 from localstack.utils.aws import aws_stack
-from localstack.services.s3 import s3_listener
+from localstack.services.s3 import s3_listener, s3_utils
 from localstack.utils.server import multiserver
 from localstack.utils.common import wait_for_port_open, get_free_tcp_port
 from localstack.services.infra import start_moto_server
 from localstack.services.awslambda.lambda_api import BUCKET_MARKER_LOCAL
+from urllib.parse import urlparse
 
 LOG = logging.getLogger(__name__)
 
@@ -172,7 +173,7 @@ def apply_patches():
         def __init__(self, *args, **kwargs):
             super(InvalidObjectState, self).__init__(
                 'InvalidObjectState',
-                'The operation is not valid for the object\"s storage class.',
+                "The operation is not valid for the object's storage class.",
                 *args,
                 **kwargs
             )
@@ -291,3 +292,16 @@ def apply_patches():
 
     s3_responses.S3ResponseInstance.is_delete_keys = types.MethodType(
         s3_response_is_delete_keys, s3_responses.S3ResponseInstance)
+
+    def parse_bucket_name_from_url(self, request, url):
+        path = urlparse(url).path
+        return s3_utils.extract_bucket_name(request.headers, path)
+
+    s3_responses.S3ResponseInstance.parse_bucket_name_from_url = types.MethodType(
+        parse_bucket_name_from_url, s3_responses.S3ResponseInstance)
+
+    def subdomain_based_buckets(self, request):
+        return s3_utils.uses_host_addressing(request.headers)
+
+    s3_responses.S3ResponseInstance.subdomain_based_buckets = types.MethodType(
+        subdomain_based_buckets, s3_responses.S3ResponseInstance)
