@@ -3,6 +3,7 @@ import re
 import sys
 import json
 import time
+import yaml
 import select
 import pkgutil
 import logging
@@ -225,6 +226,15 @@ def get_main_container_ip():
     return run(cmd).strip()
 
 
+def get_main_container_id():
+    container_name = get_main_container_name()
+    try:
+        cmd = "%s inspect -f '{{ .Id }}' %s" % (config.DOCKER_CMD, container_name)
+        return run(cmd, print_error=False).strip()
+    except Exception:
+        return None
+
+
 def get_main_container_name():
     cmd = "%s inspect -f '{{ .Name }}' %s" % (config.DOCKER_CMD, config.HOSTNAME)
     try:
@@ -361,8 +371,11 @@ def start_infra_locally():
 
 
 def validate_localstack_config(name):
+    LOG.setLevel(logging.INFO)
+
     dirname = os.getcwd()
     compose_file_name = name if os.path.isabs(name) else os.path.join(dirname, name)
+    warns = []
 
     # validating docker-compose file
     cmd = "docker-compose -f '%s' config" % (compose_file_name)
@@ -373,42 +386,66 @@ def validate_localstack_config(name):
         return False
 
     # validating docker-compose variable
-    import yaml
     with open(compose_file_name) as file:
         compose_content = yaml.full_load(file)
-    localstack_service = [service for service in compose_content['services']
-        if compose_content['services'][service]['image'] in constants.OFFICIAL_IMAGES]
-    if len(localstack_service) > 0:
-        localstack_service = localstack_service[0]
-    else:
-        raise Exception('No official docker image found. Please use one of this image: %s'
-            % (constants.OFFICIAL_IMAGES))
+    services_config = compose_content.get('services', {})
+    ls_service_name = [name for name, svc in services_config.items() if 'localstack' in svc.get('image', '')]
+    if not ls_service_name:
+        raise Exception('No LocalStack service found in config (looking for image names containing "localstack")')
+    if len(ls_service_name) > 1:
+        LOG.warning('Multiple candidates found for LocalStack service: %s' % ls_service_name)
+    ls_service_name = ls_service_name[0]
+    ls_service_details = services_config[ls_service_name]
+    image_name = ls_service_details.get('image', '')
+    if image_name.split(':')[0] not in constants.OFFICIAL_IMAGES:
+        LOG.info('Using custom image "%s", we recommend using an official image: %s' %
+            (image_name, constants.OFFICIAL_IMAGES))
 
-    network_mode = compose_content['services'][localstack_service].get('network_mode')
-    image_name = compose_content['services'][localstack_service]['image']
-    container_name = compose_content['services'][localstack_service].get('container_name') or ''
-    docker_ports = (port.split(':')[0] for port in compose_content['services'][localstack_service].get('ports', []))
+    # prepare config options
+    network_mode = ls_service_details.get('network_mode')
+    image_name = ls_service_details.get('image')
+    container_name = ls_service_details.get('container_name') or ''
+    docker_ports = (port.split(':')[0] for port in ls_service_details.get('ports', []))
     docker_env = dict((env.split('=')[0], env.split('=')[1])
-        for env in compose_content['services'][localstack_service]['environment'])
+        for env in ls_service_details.get('environment', {}))
+    edge_port = str(docker_env.get('EDGE_PORT') or config.EDGE_PORT)
+    main_container = config.MAIN_CONTAINER_NAME
 
     # docker-compose file validation cases
+
     if (docker_env.get('LAMBDA_REMOTE_DOCKER') in constants.FALSE_STRINGS and
             docker_env.get('HOST_TMP_FOLDER') in ['${TMPDIR}', None, '']):
-        LOG.warning('Make sure to properly set the "HOST_TMP_FOLDER" environment variable for the '
+        warns.append('Make sure to properly set the "HOST_TMP_FOLDER" environment variable for the '
                     'LocalStack container when using "LAMBDA_REMOTE_DOCKER=false"')
+
     if docker_env.get('PORT_WEB_UI') not in ['${PORT_WEB_UI- }', None, ''] and image_name == 'localstack/localstack':
-        LOG.warning('"PORT_WEB_UI" Web UI is now deprecated, '
+        warns.append('"PORT_WEB_UI" Web UI is now deprecated, '
                     'and requires to use the "localstack/localstack-full" image.')
-    if ('localstack_main' not in container_name) and not docker_env.get('MAIN_CONTAINER_NAME'):
-        LOG.warning('Please use "container_name: localstack_main" or add "MAIN_CONTAINER_NAME" in "environment".')
-    if docker_env.get('EDGE_PORT') and docker_env.get('EDGE_PORT') not in docker_ports:
-        LOG.warning('Using a custom edge port which is not exposed. '
-                    'You may have to add the entry to the "ports" section of the docker-compose file.')
+
+    if (main_container not in container_name) and not docker_env.get('MAIN_CONTAINER_NAME'):
+        warns.append('Please use "container_name: %s" or add "MAIN_CONTAINER_NAME" in "environment".' %
+            main_container)
+
+    def port_exposed(port):
+        for exposed in docker_ports:
+            if re.match(r'^([0-9]+-)?%s(-[0-9]+)?$' % port, exposed):
+                return True
+
+    if not port_exposed(edge_port):
+        warns.append(('Edge port %s is not exposed. You may have to add the entry '
+                    'to the "ports" section of the docker-compose file.') % edge_port)
+
     if network_mode != 'bridge':
-        LOG.warning('Network mode is not set to bridge which may cause networking issues in lambda containers. '
+        warns.append('Network mode is not set to "bridge" which may cause networking issues in Lambda containers. '
                     'Consider adding "network_mode: bridge" to you docker-compose file.')
 
-    return True
+    # print warning/info messages
+    for warning in warns:
+        LOG.warning(warning)
+    if not warnings:
+        LOG.info('Done validating config file %s - no issues found' % compose_file_name)
+        return True
+    return False
 
 
 class PortMappings(object):
@@ -589,6 +626,7 @@ def start_infra_in_docker():
         def run(self):
             self.process = run(self.cmd, asynchronous=True)
 
+    # keep this print output here for debugging purposes
     print(docker_cmd)
     t = ShellRunnerThread(docker_cmd)
     t.start()
