@@ -5,6 +5,8 @@ import base64
 import logging
 import traceback
 from urllib.parse import urlparse
+
+from moto.ec2.utils import generate_route_id
 from six import iteritems
 from moto.core import CloudFormationModel as MotoCloudFormationModel
 from moto.cloudformation import parsing
@@ -15,13 +17,14 @@ from localstack.utils.aws import aws_stack
 from localstack.constants import TEST_AWS_ACCOUNT_ID, FALSE_STRINGS
 from localstack.services.s3 import s3_listener
 from localstack.utils.common import (
-    json_safe, md5, canonical_json, short_uid, to_str, to_bytes, download, mkdir, cp_r, prevent_stack_overflow)
+    json_safe, md5, canonical_json, short_uid, to_str, to_bytes, download,
+    mkdir, cp_r, prevent_stack_overflow, start_worker_thread)
 from localstack.utils.testutil import create_zip_file, delete_all_s3_objects
 from localstack.utils.cloudformation import template_preparer
 from localstack.services.awslambda.lambda_api import get_handler_file_from_name
 from localstack.services.cloudformation.service_models import GenericBaseModel, DependencyNotYetSatisfied
 from localstack.services.cloudformation.deployment_utils import (
-    dump_json_params, select_parameters, param_defaults, remove_none_values, params_list_to_dict,
+    dump_json_params, select_parameters, param_defaults, remove_none_values,
     lambda_keys_to_lower, PLACEHOLDER_AWS_NO_VALUE, PLACEHOLDER_RESOURCE_NAME)
 
 ACTION_CREATE = 'create'
@@ -51,20 +54,6 @@ class NoStackUpdates(Exception):
     pass
 
 
-def str_or_none(o):
-    return o if o is None else json.dumps(o) if isinstance(o, (dict, list)) else str(o)
-
-
-def params_select_attributes(*attrs):
-    def do_select(params, **kwargs):
-        result = {}
-        for attr in attrs:
-            if params.get(attr) is not None:
-                result[attr] = str_or_none(params.get(attr))
-        return result
-    return do_select
-
-
 def lambda_get_params():
     return lambda params, **kwargs: params
 
@@ -183,40 +172,6 @@ RESOURCE_TO_FUNCTION = {
             'parameters': rename_params(dump_json_params(None, 'PolicyDocument'), {'PolicyDocument': 'Policy'})
         }
     },
-    'SQS::Queue': {
-        'create': {
-            'function': 'create_queue',
-            'parameters': {
-                'QueueName': ['QueueName', PLACEHOLDER_RESOURCE_NAME],
-                'Attributes': params_select_attributes(
-                    'ContentBasedDeduplication', 'DelaySeconds', 'FifoQueue', 'MaximumMessageSize',
-                    'MessageRetentionPeriod', 'VisibilityTimeout', 'RedrivePolicy', 'ReceiveMessageWaitTimeSeconds'
-                ),
-                'tags': params_list_to_dict('Tags')
-            }
-        },
-        'delete': {
-            'function': 'delete_queue',
-            'parameters': {
-                'QueueUrl': 'PhysicalResourceId'
-            }
-        }
-    },
-    'SNS::Topic': {
-        'create': {
-            'function': 'create_topic',
-            'parameters': {
-                'Name': 'TopicName',
-                'Tags': 'Tags'
-            }
-        },
-        'delete': {
-            'function': 'delete_topic',
-            'parameters': {
-                'TopicArn': 'PhysicalResourceId'
-            }
-        }
-    },
     'SecretsManager::Secret': {
         'create': {
             'function': 'create_secret',
@@ -350,6 +305,20 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
+    'Events::EventBus': {
+        'create': {
+            'function': 'create_event_bus',
+            'parameters': {
+                'Name': 'Name'
+            }
+        },
+        'delete': {
+            'function': 'delete_event_bus',
+            'parameters': {
+                'Name': 'Name'
+            }
+        }
+    },
     'Events::Rule': {
         'create': [{
             'function': 'put_rule',
@@ -387,21 +356,6 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
-    'ApiGateway::RestApi': {
-        'create': {
-            'function': 'create_rest_api',
-            'parameters': {
-                'name': 'Name',
-                'description': 'Description'
-            }
-        },
-        'delete': {
-            'function': 'delete_rest_api',
-            'parameters': {
-                'restApiId': 'PhysicalResourceId',
-            }
-        }
-    },
     'ApiGateway::Resource': {
         'create': {
             'function': 'create_resource',
@@ -430,6 +384,18 @@ RESOURCE_TO_FUNCTION = {
             'parameters': lambda_keys_to_lower()
         }
     },
+    'ApiGateway::Model': {
+        'create': {
+            'function': 'create_model',
+            'parameters': {
+                'name': 'Name',
+                'restApiId': 'RestApiId',
+            },
+            'defaults': {
+                'contentType': 'application/json'
+            }
+        }
+    },
     'ApiGateway::Deployment': {
         'create': {
             'function': 'create_deployment',
@@ -500,6 +466,12 @@ RESOURCE_TO_FUNCTION = {
                 'name': ['Name', PLACEHOLDER_RESOURCE_NAME],
                 'tags': 'Tags'
             }
+        },
+        'delete': {
+            'function': 'delete_activity',
+            'parameters': {
+                'activityArn': 'PhysicalResourceId'
+            }
         }
     },
     'KMS::Key': {
@@ -546,37 +518,6 @@ RESOURCE_TO_FUNCTION = {
                 'InstanceIds': lambda params, **kw: [kw['resources'][kw['resource_id']]['PhysicalResourceId']]
             }
         }
-    },
-    'EC2::SecurityGroup': {
-        'create': {
-            'function': 'create_security_group',
-            'parameters': {
-                'GroupName': 'GroupName',
-                'VpcId': 'VpcId',
-                'Description': 'GroupDescription'
-            }
-        },
-        'delete': {
-            'function': 'delete_security_group',
-            'parameters': {
-                'GroupId': 'PhysicalResourceId'
-            }
-        }
-    },
-    'IAM::InstanceProfile': {
-        'create': {
-            'function': 'create_instance_profile',
-            'parameters': {
-                'InstanceProfileName': 'InstanceProfileName',
-                'Path': 'Path'
-            }
-        },
-        'delete': {
-            'function': 'delete_instance_profile',
-            'parameters': {
-                'InstanceProfileName': 'InstanceProfileName'
-            }
-        }
     }
 }
 
@@ -955,10 +896,11 @@ def resolve_refs_recursively(stack_name, value, resources):
         if stripped_fn_lower == 'sub':
             item_to_sub = value[keys_list[0]]
 
+            attr_refs = dict([(r, {'Ref': r}) for r in STATIC_REFS])
             if not isinstance(item_to_sub, list):
-                attr_refs = dict([(r, {'Ref': r}) for r in STATIC_REFS])
-                item_to_sub = [item_to_sub, attr_refs]
+                item_to_sub = [item_to_sub, {}]
             result = item_to_sub[0]
+            item_to_sub[1].update(attr_refs)
 
             for key, val in item_to_sub[1].items():
                 val = resolve_refs_recursively(stack_name, val, resources)
@@ -1030,7 +972,7 @@ def resolve_refs_recursively(stack_name, value, resources):
             value_to_encode = resolve_refs_recursively(stack_name, value_to_encode, resources)
             return to_str(base64.b64encode(to_bytes(value_to_encode)))
 
-        for key, val in value.items():
+        for key, val in dict(value).items():
             value[key] = resolve_refs_recursively(stack_name, val, resources)
 
     if isinstance(value, list):
@@ -1185,6 +1127,37 @@ def delete_resource(resource_id, resources, stack_name):
             if 'NoSuchEntity' not in str(e):
                 raise
 
+    if res_type == 'AWS::EC2::VPC':
+        state = res['_state_']
+        physical_resource_id = resources[resource_id]['PhysicalResourceId'] or state.get('VpcId')
+        resources[resource_id]['PhysicalResourceId'] = physical_resource_id
+
+        if state.get('VpcId'):
+            ec2_client = aws_stack.connect_to_service('ec2')
+            resp = ec2_client.describe_route_tables(
+                Filters=[
+                    {'Name': 'vpc-id', 'Values': [state.get('VpcId')]},
+                    {'Name': 'association.main', 'Values': ['false']}
+                ]
+            )
+            for rt in resp['RouteTables']:
+                ec2_client.delete_route_table(RouteTableId=rt['RouteTableId'])
+
+    if res_type == 'AWS::EC2::Subnet':
+        state = res['_state_']
+        physical_resource_id = resources[resource_id]['PhysicalResourceId'] or state['SubnetId']
+        resources[resource_id]['PhysicalResourceId'] = physical_resource_id
+
+    if res_type == 'AWS::EC2::RouteTable':
+        ec2_client = aws_stack.connect_to_service('ec2')
+        resp = ec2_client.describe_vpcs()
+        vpcs = [vpc['VpcId'] for vpc in resp['Vpcs']]
+
+        vpc_id = res.get('Properties', {}).get('VpcId')
+        if vpc_id not in vpcs:
+            # VPC already deleted before
+            return
+
     return execute_resource_action(resource_id, resources, stack_name, ACTION_DELETE)
 
 
@@ -1564,6 +1537,7 @@ def update_resource_details(stack, resource_id, details, action=None):
     resource_type = resource.get('Type') or ''
     resource_type = re.sub('^AWS::', '', resource_type)
     resource_props = resource.get('Properties', {})
+
     if resource_type == 'ApiGateway::RestApi':
         resource_props['id'] = details['id']
 
@@ -1580,6 +1554,31 @@ def update_resource_details(stack, resource_id, details, action=None):
     if resource_type == 'IAM::InstanceProfile':
         stack.resources[resource_id]['PhysicalResourceId'] = details['InstanceProfile']['InstanceProfileName']
 
+    if resource_type == 'Events::EventBus':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['EventBusArn']
+
+    if resource_type == 'StepFunctions::Activity':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['activityArn']
+
+    if resource_type == 'ApiGateway::Model':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['id']
+
+    if resource_type == 'EC2::VPC':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['Vpc']['VpcId']
+
+    if resource_type == 'EC2::Subnet':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['Subnet']['SubnetId']
+
+    if resource_type == 'EC2::RouteTable':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['RouteTable']['RouteTableId']
+
+    if resource_type == 'EC2::Route':
+        stack.resources[resource_id]['PhysicalResourceId'] = generate_route_id(
+            resource_props['RouteTableId'],
+            resource_props.get('DestinationCidrBlock', ''),
+            resource_props.get('DestinationIpv6CidrBlock')
+        )
+
     if isinstance(details, MotoCloudFormationModel):
         # fallback: keep track of moto resource status
         stack.moto_resource_statuses[resource_id] = details
@@ -1631,6 +1630,9 @@ def add_default_resource_props(resource, stack_name, resource_name=None,
     elif res_type == 'AWS::ApiGateway::UsagePlan' and not props.get('UsagePlanName'):
         props['UsagePlanName'] = _generate_res_name()
 
+    elif res_type == 'AWS::ApiGateway::Model' and not props.get('Name'):
+        props['Name'] = _generate_res_name()
+
     elif res_type == 'AWS::DynamoDB::Table':
         update_dynamodb_index_resource(resource)
         props['TableName'] = props.get('TableName') or _generate_res_name()
@@ -1692,31 +1694,28 @@ class TemplateDeployer(object):
     def deploy_stack(self):
         self.stack.set_stack_status('CREATE_IN_PROGRESS')
         try:
-            self.apply_changes(self.stack, self.stack, stack_name=self.stack.stack_name, initialize=True)
-            self.stack.set_stack_status('CREATE_COMPLETE')
+            self.apply_changes(self.stack, self.stack, stack_name=self.stack.stack_name,
+                initialize=True, action='CREATE')
         except Exception as e:
             LOG.info('Unable to create stack %s: %s' % (self.stack.stack_name, e))
             self.stack.set_stack_status('CREATE_FAILED')
             raise
 
     def apply_change_set(self, change_set):
-        change_set.stack.set_stack_status('UPDATE_IN_PROGRESS')
+        action = 'CREATE'
+        change_set.stack.set_stack_status('%s_IN_PROGRESS' % action)
         try:
-            change_set.changes = self.apply_changes(change_set.stack, change_set, stack_name=change_set.stack_name)
-            change_set.metadata['Status'] = 'UPDATE_COMPLETE'
-            self.stack.set_stack_status('UPDATE_COMPLETE')
+            self.apply_changes(change_set.stack, change_set, stack_name=change_set.stack_name, action=action)
         except Exception as e:
             LOG.info('Unable to apply change set %s: %s' % (change_set.metadata.get('ChangeSetName'), e))
-            change_set.metadata['Status'] = 'UPDATE_FAILED'
-            self.stack.set_stack_status('UPDATE_FAILED')
+            change_set.metadata['Status'] = '%s_FAILED' % action
+            self.stack.set_stack_status('%s_FAILED' % action)
             raise
 
     def update_stack(self, new_stack):
         self.stack.set_stack_status('UPDATE_IN_PROGRESS')
         # apply changes
-        self.apply_changes(self.stack, new_stack, stack_name=self.stack.stack_name)
-        # update status
-        self.stack.set_stack_status('UPDATE_COMPLETE')
+        self.apply_changes(self.stack, new_stack, stack_name=self.stack.stack_name, action='UPDATE')
 
     def delete_stack(self):
         self.stack.set_stack_status('DELETE_IN_PROGRESS')
@@ -1886,35 +1885,56 @@ class TemplateDeployer(object):
             parameters.update({p['ParameterKey']: p for p in change_set.metadata['Parameters']})
 
         old_stack.metadata['Parameters'] = [
-            {
-                'ParameterKey': k,
-                'ParameterValue': v
-            }
+            {'ParameterKey': k, 'ParameterValue': v}
             for k, v in parameters.items() if v
         ]
 
-    def apply_changes(self, existing_stack, new_stack, stack_name, change_set_id=None, initialize=False):
+    def construct_changes(self, existing_stack, new_stack, initialize=False,
+            change_set_id=None, append_to_changeset=False):
+        from localstack.services.cloudformation.cloudformation_api import StackChangeSet
+
         old_resources = existing_stack.template['Resources']
         new_resources = new_stack.template['Resources']
-        self.init_resource_status(old_resources, action='UPDATE')
         deletes = [val for key, val in old_resources.items() if key not in new_resources]
         adds = [val for key, val in new_resources.items() if initialize or key not in old_resources]
         modifies = [val for key, val in new_resources.items() if key in old_resources]
 
-        self.apply_parameter_changes(existing_stack, new_stack)
-
-        # construct changes
         changes = []
-        contains_changes = False
         for action, items in (('Remove', deletes), ('Add', adds), ('Modify', modifies)):
             for item in items:
                 item['Properties'] = item.get('Properties', {})
-                if action != 'Modify' or self.resource_config_differs(item):
-                    contains_changes = True
                 change = self.get_change_config(action, item, change_set_id=change_set_id)
                 changes.append(change)
-                if action in ['Modify', 'Add']:
-                    self.merge_properties(item['LogicalResourceId'], existing_stack, new_stack)
+
+        # append changes to change set
+        if append_to_changeset and isinstance(new_stack, StackChangeSet):
+            new_stack.changes.extend(changes)
+
+        return changes
+
+    def apply_changes(self, existing_stack, new_stack, stack_name,
+            change_set_id=None, initialize=False, action=None):
+        old_resources = existing_stack.template['Resources']
+        new_resources = new_stack.template['Resources']
+        action = action or 'CREATE'
+        self.init_resource_status(old_resources, action='UPDATE')
+
+        # apply parameter changes to existing stack
+        self.apply_parameter_changes(existing_stack, new_stack)
+
+        # construct changes
+        changes = self.construct_changes(existing_stack, new_stack,
+            initialize=initialize, change_set_id=change_set_id)
+
+        # check if we have actual changes in the stack, and prepare properties
+        contains_changes = False
+        for change in changes:
+            res_action = change['ResourceChange']['Action']
+            resource = new_resources.get(change['ResourceChange']['LogicalResourceId'])
+            if res_action != 'Modify' or self.resource_config_differs(resource):
+                contains_changes = True
+            if res_action in ['Modify', 'Add']:
+                self.merge_properties(resource['LogicalResourceId'], existing_stack, new_stack)
         if not contains_changes:
             raise NoStackUpdates('No updates are to be performed.')
 
@@ -1922,9 +1942,27 @@ class TemplateDeployer(object):
         existing_stack.template['Outputs'].update(new_stack.template.get('Outputs', {}))
 
         # start deployment loop
-        return self.apply_changes_in_loop(changes, existing_stack, stack_name)
+        return self.apply_changes_in_loop(changes, existing_stack, stack_name, action=action, new_stack=new_stack)
 
-    def apply_changes_in_loop(self, changes, stack, stack_name):
+    def apply_changes_in_loop(self, changes, stack, stack_name, action=None, new_stack=None):
+        from localstack.services.cloudformation.cloudformation_api import StackChangeSet
+
+        def _run(*args):
+            try:
+                self.do_apply_changes_in_loop(changes, stack, stack_name)
+                status = '%s_COMPLETE' % action
+            except Exception as e:
+                LOG.debug('Error applying changes for CloudFormation stack "%s": %s %s' % (
+                    stack.stack_name, e, traceback.format_exc()))
+                status = '%s_FAILED' % action
+            stack.set_stack_status(status)
+            if isinstance(new_stack, StackChangeSet):
+                new_stack.metadata['Status'] = status
+
+        # run deployment in background loop, to avoid client network timeouts
+        return start_worker_thread(_run)
+
+    def do_apply_changes_in_loop(self, changes, stack, stack_name):
         # apply changes in a retry loop, to resolve resource dependencies and converge to the target state
         changes_done = []
         max_iters = 30
@@ -1982,14 +2020,14 @@ class TemplateDeployer(object):
         res_change = change['ResourceChange']
         action = res_change['Action']
 
-        # resolve refs in resource details
-        resolve_refs_recursively(stack.stack_name, resource, new_resources)
-
         # check resource condition, if present
         if not evaluate_resource_condition(resource, stack.stack_name, new_resources):
             LOG.debug('Skipping deployment of "%s", as resource condition evaluates to false' % resource_id)
             return
 
+        # resolve refs in resource details
+        resolve_refs_recursively(stack.stack_name, resource, new_resources)
+
         if action in ['Add', 'Modify']:
             is_deployed = self.is_deployed(resource)
             if action == 'Modify' and not is_deployed:
